# GraphMaker: Can Diffusion Models Generate Large Attributed Graphs?

## Installation

```bash
conda create -n GraphMaker python=3.8 -y
conda activate GraphMaker
pip install torch==1.12.0+cu116 torchvision==0.13.0+cu116 torchaudio==0.12.0 --extra-index-url https://download.pytorch.org/whl/cu116
conda install -c conda-forge cudatoolkit=11.6
pip install dgl==1.1.0+cu116 -f https://data.dgl.ai/wheels/cu116/repo.html
pip install pandas
pip install scikit-learn
pip install pydantic
pip install wandb
```

To compile orca.cpp for counting 4-node orbits, which will be used by an evaluation metric:

```bash
cd orca
g++ -O2 -std=c++11 -o orca orca.cpp
```

You may also need to manually set the environment variable if the program in the section below cannot find `libcusparse.so`. To search for the location of it on linux, you can use

```bash
find /path/to/directory -name libcusparse.so.11 -exec realpath {} \;
```

where `/path/to/directory` is the directory you want to search.

Say this returns `home/miniconda3/envs/GraphMaker/lib/libcusparse.so.11`. Then you need to manually specify the environment variable as follows.

```bash
export LD_LIBRARY_PATH=home/miniconda3/envs/GraphMaker/lib:$LD_LIBRARY_PATH
```

## Usage

### Training

```bash
python train_X_E.py --dataset X
python train_X_E_asym.py --dataset X
python train_E.py --dataset X
```

where `X` can be `cora`, `citeseer`, `amazon_photo`, or `amazon_computer`.

We use `wandb` to log the learning curves, but you can choose not to use it.

### Sampling and Evaluation

```bash
python sample.py --model_path Y
```

where `Y` is the path to a model saved from `Training`.
